package com.student.details.services;
import com.student.details.exceptions.ResourceNotFoundException;
import com.student.details.models.Student;
import com.student.details.repositories.StudentRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.net.HttpURLConnection;
import java.net.URL;
import org.json.JSONObject;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;


@Service
public class StudentService {

    @Autowired
    StudentRepository studentRepository;

    public List<Student> findAllStudents() {
        return studentRepository.findAll();
    }

    public Student addStudent(Student student) {
        return studentRepository.save(student);
    }

    public Optional<Student> getStudentById(Long studentId) {
        return studentRepository.findById(studentId);
    }

    public Optional<Student> getStudentByEmail(String email) {
        return studentRepository.findStudentByEmail(email);
    }

    public Student updateStudent(Long studentId, Student studentDetails) throws ResourceNotFoundException {
        Student student = studentRepository.findById(studentId)
                .orElseThrow(() -> new ResourceNotFoundException("Student id: " + studentId + " not found"));

        student.setFirstName(studentDetails.getFirstName());
        student.setLastName(studentDetails.getLastName());

        Student updatedStudent = studentRepository.save(student);
        return updatedStudent;

    }

    public Map<String, Boolean> deleteStudent(Long studentId) throws ResourceNotFoundException {
        Student student = studentRepository.findById(studentId)
                .orElseThrow(() -> new ResourceNotFoundException("Student id: " + studentId + " not found"));

        studentRepository.delete(student);
        Map<String, Boolean> response = new HashMap<String, Boolean>();
        response.put("deleted", true);
        return response;
    }

    public Student getStudentAddressFromEmail(Student student){
        //get students email for api call

        String email = student.getEmail();
        String apiUrl = String.format("url/students/%s", email);

        //make API call other microservice's Rest API
        JSONObject response = getJSONObjectFromAPICall(apiUrl);

        //make new address object with JSON response object

        /*
        Address address;
        address.setStreet(res.get("street"));
        ...
         */


        //add it to our student's address field!

        /*
       student.setAddress(address);
         */

        return student;


    }

    public JSONObject getJSONObjectFromAPICall(String apiUrl){
        HttpURLConnection connection;
        StringBuffer responseContent = new StringBuffer();
        BufferedReader reader;
        String line;

        try {

            URL url = new URL(apiUrl);
            connection = (HttpURLConnection) url.openConnection();

            //setup
            connection.setRequestMethod("GET");
            connection.setConnectTimeout(5000);
            connection.setReadTimeout(5000);

            int status = connection.getResponseCode();

            //read and write
            if (status > 299) {
                reader = new BufferedReader(new InputStreamReader(connection.getErrorStream()));
            } else {
                reader = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            }
            while((line = reader.readLine()) != null){
                responseContent.append(line);
            }

            reader.close();


        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        JSONObject res = new JSONObject(responseContent.toString());
        return res;

    }

}
